/**
 * @file	mainframe.cpp
 * @brief	mainframe source file 
 * @author	la9527
 * @date	2004-08-22
 */
#include <sys/types.h>
#include <signal.h>

#include "define.h"
#include "mainframe.h"
#include "mlslocale.h"
#include "dialog.h"
#include "configure.h"

namespace MLS {

MainFrame *g_MainFrame;

MainFrame::MainFrame()
	: _tKeybind(g_Keybind)
{
	_bSplit  = false;
	_nActive = 0;
	
	UpdateConfig();
}

MainFrame::~MainFrame()
{
	SaveConfig();
}

void MainFrame::UpdateConfig( )
{
	_Func[0].addEntry(config.GetValue("FuncF1"));
	_Func[1].addEntry(config.GetValue("FuncF2"));
	_Func[2].addEntry(config.GetValue("FuncF3"));
	_Func[3].addEntry(config.GetValue("FuncF4"));
	_Func[4].addEntry(config.GetValue("FuncF5"));
	_Func[5].addEntry(config.GetValue("FuncF6"));
	_Func[6].addEntry(config.GetValue("FuncF7"));
	_Func[7].addEntry(config.GetValue("FuncF8"));
	_Func[8].addEntry(config.GetValue("FuncF9"));
	_Func[9].addEntry(config.GetValue("FuncF10"));
	_Func[10].addEntry(config.GetValue("FuncF11"));
	_Func[11].addEntry(config.GetValue("FuncF12"));

	_bSplit = config.GetBool("SplitWindow");
	
}

void MainFrame::SaveConfig()
{
	config.SetBool("SplitWindow", _bSplit);
	
}


/// @brief		consol 초기화 작업.
bool MainFrame::Init()
{
	initscr();     // nCurses 시작
	start_color(); // 컬러 가능하게 
	
	nonl();        // 엔터 처리?
	//raw();		   // Ctrl+C : Signal무시 가능하게
	cbreak();      // nCurses : 즉시 입력이 가능하도록 설정
	
	noecho();	   // nCurses : echo모드 해제
	keypad(stdscr, TRUE);  // keypad 활성화
	
	curs_set(0); // 커서를 보이지 않게 한다.

	// 색깔 쌍을 초기화 한다. 0 ~64개 까지
	for (int t=0; t<8; t++)
		for (int t2=0; t2<8; t2++)
	{
		if (t==0 && t2==0) continue;
		init_pair(t*8+t2, t, t2);
	}
		
	// curses 마우스 설정		
	mousemask( BUTTON1_CLICKED | BUTTON2_CLICKED | BUTTON_SHIFT | BUTTON_CTRL,
		NULL);
	mouseinterval(30);
	
	// line 설정
	Set_Line(e_nBoxLineCode);
	
	_bShell = false;
	setcol(g_Color.Default);		
	bkgd(' ');

	return true;
}

void MainFrame::Destroy()
{
	clear();
	refresh();
	
	// . keypad 비활성화
	keypad(stdscr, FALSE);  // . 라인단위로 입력이 이루어지도록 설정	
	nocbreak();
	//noraw();
	curs_set(1); // 커서를 다시 보이게 한다.
	noecho();
	endwin();   // . nCurses 해제
}

void MainFrame::SetSplitWindow( )
{
	if (_bSplit)
	{
		_tPanel[0].SetWindow(_win1, (COLS-2)/2, LINES-6);
		_tPanel[1].SetWindow(_win2, (COLS-1)/2, LINES-6);		
	}
	else
	{
		_tPanel[_nActive].SetWindow(_win0, COLS, LINES-6);	
	}
}

void MainFrame::Draw()
{
	int i;
	//////////////////. Function Draw
	
	setcol(g_Color.Func);
	move(0,0);
	hline(' ', COLS);
	
	int	pos;
	int f = COLS / 8;
	int	bound = 7-COLS % 8;
	
	if (!bound)
		f++;

	// Funtion panel draw
	for (i=0; i<8; i++)
	{
		if (!bound || i < bound)
			pos = i*f;
		else
			pos = i*(f+1) - bound + 1;
		
		mvprintw(0, 2+pos,"%s", _Func[i+1].getName());
	}
	
	setcol(g_Color.FuncA);
	mvprintw (0, 0, "F");
	for (i=0; i<8; i++)
	{
		if (!bound || i < bound)
			pos = i*f;
		else
			pos = i*(f+1) - bound + 1;		
		mvprintw(0, 1+pos, "%d", i+2);
	}
	
	
	if (g_Color.Func.back >=8 )
		attron(A_BLINK);

	attron(COLOR(COLOR_BLACK, g_Color.Func.back%8));
	for (i=1; i<8;i++)
	{
		if (!bound || i < bound)
			pos = i*f;
		else
			pos = i*(f+1) - bound + 1;
		mvaddch(0, pos, VLINE);
	}
	

	////////////////// . PATH Draw
	setcol(g_Color.Stat);
	move(1, 0); hline(' ', COLS);
	if (_bSplit)
	{
		mvprintw(1, 0, "%s", _tPanel[0].GetPathView().c_str());
		mvprintw(1, COLS/2+1, "%s", _tPanel[1].GetPathView().c_str());
	}
	else
	{
		mvprintw(1, 0, "%s", _tPanel[_nActive].GetPathView().c_str());
	}
	

	////////////// Panel Draw
	if (_bSplit)
	{
		_tPanel[0].Draw(_nActive == 0);
		_tPanel[1].Draw(_nActive == 1);
	}
	else
	{
		_tPanel[_nActive].Draw(true);
	}

	//////// 상태선 그리기..
	_tPanel[_nActive].DrawStatus();

	//////// 2단일 경우. 중간의 꺾은선 출력.
	setcol(g_Color.Line);
	if (_bSplit)
	{
		for (int t=0; t<LINES-6; t++)
		{
			move(3 + t, (COLS-2)/2);
			addch(VLINE);
			addch(VLINE);
		}
		
		move(2, (COLS-2)/2);
		addch(TTEE);
		addch(TTEE);
		
		move(LINES-3, (COLS-2)/2);
		addch(BTEE);
		addch(BTEE);
	}
	
	////////////////////
	
	refresh();
}

void MainFrame::Refresh()
{
	wresize(_win0, LINES-6, COLS);
	wresize(_win1, LINES-6, (COLS-2)/2);
	wresize(_win2, LINES-6, (COLS-1)/2);
	
	mvwin(_win1, 3, 0);
	mvwin(_win2, 3, (COLS+2)/2);
	
	Draw();
	SetSplitWindow();
}

///	@brief 메인프레임 수행
void MainFrame::Do(
	bool bMcd			///< MCD 실행 여부.
	)
{

	_win0 = newwin(LINES-6, COLS, 3, 0);
	_win1 = newwin(LINES-6, (COLS-2)/2, 3, 0);
	_win2 = newwin(LINES-6, (COLS-1)/2, 3, (COLS+2)/2);
	
	_bExit = false;
	
	// 메뉴
	_tMenu.Create();
	_tCommand.Init();
		
	_tPanel[0].Read(".");
	_tPanel[1].Read(".");
	_tCommand.SetPanel(&_tPanel[0]);
	
	SetSplitWindow();
	
	refresh();
	
	if (bMcd)
	{
		_tCommand.Execute(_Func[9][0]);
	}
	
	// ESC 딜레이 줄이기.
	ESCDELAY = 10;
	
	// 에러 번호 체크
	int nErrno = 0;
	
	while(!_bExit)
	{
		// 패널 화면을 그린다.
		Draw();
		
		// 쉘을 실행하면 계속 실행하게 한다.
		if (_bShell == true) 
		{	
			_tCommand.Execute("<shell>");
			continue;
		}
		
		// 키를 입력 받는다.
		int nKey = getch();
		
		LOG("KEY NUM :: %d", nKey);
		
		// 쓰레드를 이용해 키 수신 
		/*
		int nKey = (int)_pMlsMutex->Wait();
		_pMlsMutex->WaitEnd();
		*/
		
		// 콘솔에 따라서 백그라운드로 남아 있는 것을 잡기 위한 방법
		if (nKey == -1)
		{
			nErrno++;
			if (nErrno == 10)
			{
				kill(getpid(), 3); // Signal Quit로 종료
			}

			continue;
		}
		else
		{
			nErrno = 0;
		}
		
		if (nKey == 3)
		{
			nKey = getch();
			_tCommand.Execute("<copyclip>");
			continue;
		}
		
		// 리사이즈 		
		if (nKey == KEY_RESIZE)
		{
			Refresh();
			continue;
		}
		
		// 마우스		
		if (nKey == KEY_MOUSE)
		{
			MouseProc();
			continue;
		}	
						
		// Searching..
		if (_tPanel[_nActive].SearchProcess(nKey))
		{
			continue;
		}
		
		// Function key
		if (KEY_F(1) <= nKey && nKey <=KEY_F(12))
		{
			if (_Func[nKey-KEY_F(1)].size()>=1)
			{
				_tCommand.Execute(_Func[nKey-KEY_F(1)][0]);
			}
			continue;
		}
		
		// Execute 
		if (nKey != 27)
		{
			string	command = _tKeybind.GetCommand(nKey);
			_tCommand.Execute(command);
		}
		else
		{
			// Alt + 문자는 이렇게 받는다. 
			nodelay(stdscr, TRUE);
			nKey = getch();
			nodelay(stdscr, FALSE);
			
			if (nKey != ERR)
			{
				_tCommand.Execute(_tKeybind.GetMetaCommand(nKey));
			}
			else
			{
				_tCommand.Execute(_tKeybind.GetCommand(27));
			}
		}
	}
}

/// @brief	NextWindow
void MainFrame::NextWindow()
{
    /// @todo implement me
	if (_bSplit)
	{
		_nActive = (_nActive + 1) %2;
		_tCommand.SetPanel(&_tPanel[_nActive]);
		
		// 디렉토리 이동
		chdir(_tPanel[_nActive].GetPathView().c_str());
	}
}

};  // namespace

void MLS::MainFrame::RefreshPanel( )
{
	if (_bSplit)
	{
		_tPanel[0].Refresh();
		_tPanel[1].Refresh();		
	}
	else
	{
		_tPanel[_nActive].Refresh();
	}
	chdir(_tPanel[_nActive].GetPathView().c_str());
}

#define	MENU_XPOS				0		///< 화면상의 Menu Y 위치
#define	FUNCTIONKEY_TOTALCOUNT	8		///< 화면상의 Functionkey 개수
#define	FILELIST_MIN_POS		3		///< FileList 시작 y 위치. 

/**
 * @brief	마우스 이벤트 처리
 */
void MLS::MainFrame::MouseProc()
{
	MEVENT event;
	getmouse(&event);
	
	// 마우스 이벤트가 있으면 종료
	if (!(event.bstate & BUTTON1_CLICKED))
	{
		return;
	}

	// 마우스가 메뉴를 눌렸을 경우
	if(event.y == MENU_XPOS)
	{
		int functionkey;
		// 어느 메뉴를 눌렀는지 확인하여 실행
		functionkey = XPositionToFunctionkey(event.x);
		if(KEY_F(1) < functionkey && functionkey <= KEY_F(9))
		{
			_tCommand.Execute(_Func[functionkey-KEY_F(1)][0]);
		}

	}
	// 마우스가 파일List를 눌렀을경우
	else if(FILELIST_MIN_POS <= event.y)
	{
		int	panelIndex;
		// 해당 패널 구하기
		panelIndex = XPositionToFilePanel(event.x);

		// x, y 위치를 패널에 맞게 지정
		event.x = event.x - panelIndex * ((int)COLS/2);
		event.y = event.y - 3;

		// 해당 패널 활성화.
		if(panelIndex != _nActive)
		{
			NextWindow();
		}
	
		// 메시지 전달
		_tPanel[panelIndex].MouseProc(event);
	}
}


/**
 * @brief	화면상의 X position을 가지고 Function key을 얻는다
 * @return	Function key (KEY_F(2) ~ KEY_F(8))
 **/
int MLS::MainFrame::XPositionToFunctionkey(
	const int xPosition		///<< 화면상의 X position
	)
{
	int		functionInterval = COLS / FUNCTIONKEY_TOTALCOUNT;
	int		overspace = (FUNCTIONKEY_TOTALCOUNT - 1) - 
		(COLS % FUNCTIONKEY_TOTALCOUNT);
	int		pos;
	int		oldPos = 0;
	int		functionXPos[FUNCTIONKEY_TOTALCOUNT] = {0,}; 

	if (overspace == 0)
		functionInterval++;
		

	int		functionkey = -1;
	for (int i=1; i<=FUNCTIONKEY_TOTALCOUNT; i++){
		if (!overspace || i < overspace) {
			pos = i * functionInterval;
		}
		else {
			pos = i*(functionInterval+1) - overspace + 1;
		}

		if(oldPos < xPosition && xPosition < pos) {
			functionkey = KEY_F(i+1);
			break;
		}

		oldPos = pos;
	}

	return functionkey;
}

/// @brief	window split switching Function.
void MLS::MainFrame::SwitchSplit()
{
	_bSplit = !_bSplit; 
}



/// @brief	x 값으로 해당 패널을 찾는 함수.
/// @return	해당 패널 index.
int MLS::MainFrame::XPositionToFilePanel(
	const int		x			///< 화면 X position
	)
{
	if(!_bSplit || x < COLS/2)
		return 0;

	return 1;
}
